package com.infine.blog.equals.color;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class RgbColor {

	private final int green;
	private final int red;
	private final int blue;
	// pooling
	private static Map<Integer, RgbColor> pool = new ConcurrentHashMap<Integer, RgbColor>();

	/**
	 * Values must be between 0 and 255.
	 * 
	 * @param green
	 * @param red
	 * @param blue
	 * @return
	 */
	public static RgbColor getColor(int green, int red, int blue) {
		if (!isAllowedColorValue(green) || !isAllowedColorValue(red) || !isAllowedColorValue(blue)) {
			throw new IllegalArgumentException("Allowed color value is between 0 and 255");
		}
		// same key as hashcode
		int key = green;
		key = key << 8;
		key = key | red;
		key = key << 8;
		key = key | blue;
		if (pool.containsKey(key)) {
			return pool.get(key);
		}
		RgbColor color = new RgbColor(green, red, blue);
		pool.put(key, color);
		return pool.get(key);
	}

	private static boolean isAllowedColorValue(int color) {
		if (color < 0 || color > 255) {
			return false;
		}
		return true;
	}

	private RgbColor(int green, int red, int blue) {
		super();
		this.green = green;
		this.red = red;
		this.blue = blue;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		RgbColor other = (RgbColor) obj;
		if (blue != other.blue)
			return false;
		if (green != other.green)
			return false;
		if (red != other.red)
			return false;
		return true;
	}

	@Override
	public int hashCode() {
		int hash = green;
		hash = hash << 8;
		hash = hash | red;
		hash = hash << 8;
		hash = hash | blue;
		return hash;
	}

	// ...other methods
}
